EHH <- function(snp) {
# the probability that two chromosomes are the same	
	Nsam <- length(snp)
	hfs <- table(snp) # haplotype frequency spectrum
	EHH <- sum( hfs * (hfs-1) /2 ) / (Nsam * (Nsam-1) / 2)
	return(EHH)
}

format <- function(line) {
	Nline <- length(line)
	format = "wrong"
	
	i=1;
	while( length(grep("^\\s*$", line[i])) > 0 && i <= Nline ) { i = i+1 } # skip the empty lines
	
	if (length(grep("^>",line[i])) > 0 ) {
		format = "fasta"
	} else if (length(grep("ms",line[i])) > 0 ) {
		format = "ms"
	} 

	if( format=="wrong" ) {
		print("Wrong format! The format can only be ms/msms-result-file format or fasta format.")
	}

	return(format)
}

computing <- function(pos, snp, win_size, win_step) {
	if(length(pos)==0) {
		return( rep( 0, 7*((1-win_size)/win_step+1) + 2*((1/2-win_size)/win_step+1) ) )
	}

	a1=0; a2=0; b1=0; b2=0; c1=0; c2=0; e1=0; e2=0
	Nsam <- length(snp)
	Nseg <- nchar(snp[1])
	for (i in 1:(Nsam-1)) {
		a1 = a1+1.0/i
		a2 = a2+1.0/i/i
	}
	b1 = (Nsam+1.0)/3.0/(Nsam-1.0)
	b2 = 2.0*(Nsam*Nsam+Nsam+3.0)/9.0/Nsam/(Nsam-1.0)
	c1 = b1-1.0/a1
	c2 = b2-(Nsam+2.0)/a1/Nsam*1.0+a2/a1/a1
	e1 = c1/a1
	e2 = c2/(a1*a1+a2)
	
	stat <- c()

	for (k in seq(win_size/2, 1-win_size/2, win_step)) {
		theta_w = 0
		theta_pi = 0
		theta_h = 0
		theta_l = 0
		Tajima_D = 0
		Fay_Wu_H = 0
		S = rep(0,Nsam)
		for(j in 1:Nseg) {
			if ( k-win_size/2 < pos[j] && pos[j] <= k+win_size/2 ) {
				
				Nderived = 0
				for (i in 1:Nsam) if(substr(snp[i],j,j)=="1") Nderived = Nderived+1
				if(Nderived <= Nsam-1) S[Nderived] = S[Nderived]+1
			}
		}
		MFDM = 1
		for(j in 1:(Nsam-1)) {
			if ( S[j] > 0 ) { MFDM = j; }
			theta_w = theta_w + S[j]
			theta_pi = theta_pi + 2*S[j]*j*(Nsam-j)
			theta_h = theta_h + 2*S[j]*j*j
			theta_l = theta_l + S[j]*j
		}

		theta_w = theta_w / a1;
		theta_pi = theta_pi / (Nsam*(Nsam-1))
		theta_h = theta_h / (Nsam*(Nsam-1))
		theta_l = theta_l / (Nsam-1)
		if (theta_w !=0) {
			Tajima_D = (theta_pi-theta_w)/sqrt(e1*theta_w*a1+e2*theta_w*a1*(theta_w*a1-1));
			Fay_Wu_H = (theta_pi-theta_l) / +
			sqrt((Nsam-2)*theta_w/6.0/(Nsam-1)+ +
			(18.0*Nsam*Nsam*(3.0*Nsam+2)*(a2+1.0/Nsam/Nsam)- +			# a2+1.0/Nsam/Nsam = "a2" when sample size = Nsam +1
			(88.0*Nsam*Nsam*Nsam+9.0*Nsam*Nsam-13.0*Nsam+6.0))* +
			(theta_w*a1*(theta_w*a1-1)/(a1*a1+a2))/(9.0*Nsam*(Nsam-1)*(Nsam-1)));	
		} else {
			Tajima_D = 0
			Fay_Wu_H = 0
		}
		stat <- c(stat, theta_w, theta_pi, theta_h, theta_l, Tajima_D, Fay_Wu_H, MFDM)
	}
	
	rmp = Nseg + 1	# right middle position, in case that lmp == Nseg, then rmp = Nseg + 1
	for ( j in 1:Nseg) {
		lmp = j	# left middle position, in case that lmp == Nseg
		if ( pos[j] >= 1/2 ) {
			lmp = j - 1
			rmp = j
			break
		}
	}

	iHH = 0
	if ( lmp < 1 ) {
		for ( j in 1:( (1/2-win_size)/win_step + 1) ) { stat <- c(stat, iHH) }
	} else {
		win_anchor_point = 1/2 - win_size;
		while ( pos[lmp] <= win_anchor_point) {
			stat <- c(stat, iHH)
			win_anchor_point = win_anchor_point - win_step
		}
		for ( j in lmp:1 ) {
			iHH = iHH + EHH(substr(snp, j, lmp))		
			if ( j==1 ) {
				while ( win_anchor_point > -0.00001 ) {
					stat <- c(stat, iHH)
					win_anchor_point = win_anchor_point - win_step
				}
			} else {
				while ( pos[j-1] < win_anchor_point ) {
					stat <- c(stat, iHH)
					win_anchor_point = win_anchor_point - win_step
				}
			}
		}
	}

	iHH = 0
	if ( rmp > Nseg ) {
		for ( j in 1:( (1/2-win_size)/win_step + 1) ) { stat <- c(stat, iHH) }
	} else {
		win_anchor_point = 1/2 + win_size;
		while ( pos[rmp] >= win_anchor_point) {
			stat <- c(stat, iHH)
			win_anchor_point = win_anchor_point + win_step
		}
		for ( j in rmp:Nseg ) {
			iHH = iHH + EHH(substr(snp, rmp, j))
			if ( j == Nseg ) {
				while ( win_anchor_point < 1.00001 ) {  
				# due to sum error (e.g. sum of 100 0.01s: 0.01+0.01+...+0.01 != 1), <= 1 sometimes does not work here
					stat <- c(stat, iHH)
					win_anchor_point = win_anchor_point + win_step
				}
			} else {
				while ( pos[j+1] > win_anchor_point ) {
					stat <- c(stat, iHH)
					win_anchor_point = win_anchor_point + win_step
				}
			}	
		}
	}
	return(stat)
}

statComputing <- function(file, win_size = 0.2, win_step = 0.2, y = 0 ) {
	
	head <- c()
	for ( j in 1:((1-win_size)/win_step+1) ) {
		head <- c(head, paste("theta_w",j,sep="") )
		head <- c(head, paste("theta_pi",j,sep="") )
		head <- c(head, paste("theta_h",j,sep="") )
		head <- c(head, paste("theta_l",j,sep="") )
		head <- c(head, paste("Tajima_D",j,sep="") )
		head <- c(head, paste("Fay_Wu_H",j,sep="") )
		head <- c(head, paste("MFDM",j,sep="") )
	}

	for ( j in ( floor((1/2-win_size)/win_step)+1):1 ) {
		head <- c(head, paste("iHH_l",j,sep="") )
	}

	for ( j in 1:(floor((1/2-win_size)/win_step) +1) ) {
		head <- c(head, paste("iHH_r",j,sep="") )
	}

	all_stat <- rep(0,length(head))
	names(all_stat) <- head

	line <- readLines(con=file)
	Nline <- length(line)

	format = format(line)
	rep = 1

	i <- 1
	while( length(grep("^\\s*$", line[i])) > 0 && i <= Nline ) { i = i+1 } # skip the empty lines

	if( format=="ms" ) {
		
		while(i <= Nline) {
			if (length(grep("ms",line[i])) > 0 ) {
			#if(substr(line[i],1,2)=="ms" || substr(line[i],1,4)=="./ms"){	
				arr <- strsplit(line[i], "\\s+")
				#n <- strtoi(arr[[1]][2])
				n <- as.numeric(arr[[1]][2])
				#rep <- strtoi(arr[[1]][3])		
				rep <- as.numeric(arr[[1]][3])		

				i <- i+1
					
				for (j in 1:rep) {
					#print(j)
					while(substr(line[i],1,2)!="//"){
						i <- i+1
					}
					i <- i+1	# line[i]: "segsites"
					arr <- strsplit(line[i], "\\s+")
					if(arr[[1]][2]==0) {
						stat <- computing(integer(0), 0, win_size, win_step)
					} else {
						i <- i+1	# line[i]: "positions"
						pos <- line[i]
						pos = strsplit(pos, "\\s+")[[1]]
						pos = as.numeric(pos[2:length(pos)])
						snp <- c()
						for (k in 1:n){
							i <- i+1
							snp <- c(snp, line[i])
						}

						stat <- computing(pos, snp, win_size, win_step)
					}
					
					all_stat <- rbind(all_stat, stat)
				}
			}
			i <- i+1;
		}
	} else if(format=="fasta") {
		outg = ""
		seq = c()
		while(i <= Nline) {
			if ( length(grep(">",line[i])) > 0 ) { 
				outg_indicator = 0
				if ( length(grep("outgroup",line[i],ignore.case=TRUE)) > 0 ) { outg_indicator = 1 } 
				this_seq= ""
				i <- i + 1
				while ( length(grep(">",line[i])) == 0 && length(grep("^\\s+$",line[i])) == 0 && i <= Nline) {
					arr <- strsplit(line[i], "\\s+")
					this_seq <- paste(this_seq, arr[[1]][1], sep="")
					i <- i + 1
				}
				if (outg_indicator == 1) {
					outg = this_seq
				} else {
					seq <- c(seq, this_seq)
				}
			} else {
				i <- i + 1
			}
		}

		len <- nchar(outg)
		Nsam <- length(seq)

		same = rep(1, len)

		snp_all = rep("", Nsam)
		for (i in 1:Nsam) {
			for ( j in 1:len ) {
				if ( substr(seq[i],j,j) ==  substr(outg,j,j) ) {
					snp_all[i] <- paste(snp_all[i], "0", sep="")
				} else {
					snp_all[i] <- paste(snp_all[i], "1", sep="")
				}
				if ( i > 1 && substr(seq[i],j,j) != substr(seq[1],j,j) ) {
					same[j] = 0	# this is a segregating site, all the samples are not the same at this site
				}
			}
		}

		snp = rep("", Nsam)
		pos = c()
		for ( j in 1:len) {
			if (same[j] == 0) {
				pos <- c(pos, j/len)
				for ( i in 1:Nsam ) {
					snp[i] <- paste(snp[i], substr(snp_all[i],j,j), sep="")
				}
			}
		}

		stat <- computing(pos, snp, win_size, win_step)
		
		all_stat <- rbind(all_stat, stat)
		
	} else {
		print("Wrong format! Format can only be 'ms', 'msms', or 'fasta'.")
	}
	
	all_stat <- cbind(all_stat, y=rep(y,rep+1) )
	all_stat <- as.data.frame(all_stat)
	all_stat <- all_stat[2:(rep+1),]
	rownames(all_stat) <- 1:rep
	return(all_stat)
}

