\name{predicting}
\alias{predicting}
\alias{predicting}
\title{Predicting Sample(s)}
\description{
	Predicting sample(s) with the summary statistics by the predictor 
	trained by \code{training}.
}
\usage{
predicting(predictor, stat, auto_classify=TRUE)
}
\arguments{
	\item{predictor}{ The predictor produced by \code{training}. }
	\item{stat}{ A data frame containing some samples represented by different summary 
	statistics and may or may not contain a y variable. It doesn't matter what value y is here.
	All the statistics used in \code{stat} must be the same as the \code{stat} used in \code{training}. }
	\item{auto_classify}{ If the \code{y} variable is a binary variable(0 or 1), then 
	\code{auto_classify} = TRUE will change all predicted values larger than(>) 0.5 to 1, and 
	change the values smaller than(<=) 0.5 to 0.}
}

\value{
	This function returns the predicted \code{y} values for all the tested samples. 
}

\details{
	The argument \code{auto_classify} is default set to be TRUE. So if your \code{y} variable is 
	not a binary variable with values 0 and 1, you may need to set auto_classify to FALSE.
}

\author{
	Kao Lin, Zongfeng Yang and Haipeng Li
}

\references{
	P. Buehmann and T. Hothorn(2007) Boosting algorithms: regularization, prediction, and 
	model fitting. Stat. Sci. 22: 477-505.
}

\examples{	
# data
ms <- system.file("extdata/ms.out", package="evolBoosting")
msms <- system.file("extdata/msms.out", package="evolBoosting")
msms_for_test <- system.file("extdata/msms_for_test.out", package="evolBoosting")
fasta <- system.file("extdata/test.fasta", package="evolBoosting")

# training
train1 <- statComputing(ms, 0.2, 0.2, y = 0)
train2 <- statComputing(msms, 0.2, 0.2, y = 1)
train <- rbind(train1, train2)
predictor <- training(train)

# predicting
test1 <- statComputing(msms_for_test, 0.2, 0.2)
test2 <- statComputing(fasta, 0.2, 0.2)

result1 <- predicting(predictor, test1)
result2 <- predicting(predictor, test2)
}
