\name{training}
\alias{training}
\alias{training}
\title{Training the Predictor}
\description{
	Training the predicotr from the file containing summary statistics and y variable.
	Then this predictor can be used to predict unknown y variable from new coming
	statistics.
}
\usage{
training(stat, y="y", coef=FALSE)
}
\arguments{
	\item{stat}{A data frame containing some samples represented by different summary 
	statistics and a \code{y} variable indicating the samples' categories(selected or neutral).}
	\item{y}{A string specify the name of the variable which we want to learn how to 
	predict it from the summary statistics in \code{stat}. The default value is "y". If \code{stat} was
	computed via \code{statComputing}, then the default value works.}
	\item{coef}{A logical value. If ture then the standardized coefficients of each summary statistics will be shown.}
}

\value{
	A predictor which can be used in \code{predicting}.
}

\details{
	This algorithm implements boosting algorithm(Buehmann and Hothorn 2007) to learn how 
	to predict \code{y} variable (selection or neutral evolution) from summary statistics.
	
	L2 loss, that is, square error loss(Buehmann and Hothorn 2007), is used in boosting.

	In \code{stat}, one of the column must be the {y} variable.

	The \code{y} variable should be 0 or 1. There should be several samples with \code{y} = 0 and several
	samples with \code{y} =1.

	This function is intended to handle the value produced by \code{statComputing}, but 
	one also can use user-specified data frame.
}

\author{
	Kao Lin, Zongfeng Yang and Haipeng Li
}

\references{
	P. Buehmann and T. Hothorn(2007) Boosting algorithms: regularization, prediction, and 
	model fitting. Stat. Sci. 22: 477-505.
}

\examples{
# data
ms <- system.file("extdata/ms.out", package="evolBoosting")
msms <- system.file("extdata/msms.out", package="evolBoosting")

# training
train1 <- statComputing(ms, 0.2, 0.2, y = 0)
train2 <- statComputing(msms, 0.2, 0.2, y = 1)
train <- rbind(train1, train2)
}
